"""initial

Revision ID: 4f47b5e0d6b8
Revises: 
Create Date: 2022-07-08 22:44:40.927485

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "4f47b5e0d6b8"
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "media",
        sa.Column("id", sa.String(length=32), nullable=False),
        sa.Column("hash", sa.String(length=128), nullable=False),
        sa.Column("repopath", sa.String(length=272), nullable=False),
        sa.Column("mime", sa.String(length=32), nullable=True),
        sa.Column("filename", sa.String(length=255), nullable=False),
        sa.Column("timestamp", sa.DateTime(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_media_filename"), "media", ["filename"], unique=False)
    op.create_index(op.f("ix_media_hash"), "media", ["hash"], unique=False)
    op.create_index(op.f("ix_media_repopath"), "media", ["repopath"], unique=False)
    op.create_index(op.f("ix_media_timestamp"), "media", ["timestamp"], unique=False)
    op.create_table(
        "media_collection",
        sa.Column("id", sa.String(length=32), nullable=False),
        sa.Column("name", sa.String(length=256), nullable=False),
        sa.Column("first_media", sa.DateTime(), nullable=False),
        sa.Column("last_media", sa.DateTime(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_media_collection_first_media"),
        "media_collection",
        ["first_media"],
        unique=False,
    )
    op.create_index(
        op.f("ix_media_collection_last_media"),
        "media_collection",
        ["last_media"],
        unique=False,
    )
    op.create_index(
        op.f("ix_media_collection_name"), "media_collection", ["name"], unique=False
    )
    op.create_table(
        "setting",
        sa.Column("key", sa.String(length=32), nullable=False),
        sa.Column("val", sa.String(length=256), nullable=True),
        sa.PrimaryKeyConstraint("key"),
    )
    op.create_table(
        "media_col_item",
        sa.Column("id", sa.String(length=32), nullable=False),
        sa.Column("media_collection_id", sa.String(length=32), nullable=False),
        sa.Column("media_col_item", sa.String(length=32), nullable=False),
        sa.ForeignKeyConstraint(
            ["media_col_item"],
            ["media.id"],
        ),
        sa.ForeignKeyConstraint(
            ["media_collection_id"],
            ["media_collection.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_media_col_item_media_col_item"),
        "media_col_item",
        ["media_col_item"],
        unique=False,
    )
    op.create_index(
        op.f("ix_media_col_item_media_collection_id"),
        "media_col_item",
        ["media_collection_id"],
        unique=False,
    )
    op.create_table(
        "media_gps",
        sa.Column("media_id", sa.String(length=32), nullable=False),
        sa.Column("lat", sa.Float(), nullable=False),
        sa.Column("lon", sa.Float(), nullable=False),
        sa.ForeignKeyConstraint(
            ["media_id"],
            ["media.id"],
        ),
        sa.PrimaryKeyConstraint("media_id"),
    )
    op.create_index(op.f("ix_media_gps_lat"), "media_gps", ["lat"], unique=False)
    op.create_index(op.f("ix_media_gps_lon"), "media_gps", ["lon"], unique=False)
    op.create_table(
        "media_hashtag",
        sa.Column("id", sa.String(length=32), nullable=False),
        sa.Column("media_id", sa.String(length=32), nullable=False),
        sa.Column("hashtag", sa.String(length=128), nullable=False),
        sa.ForeignKeyConstraint(
            ["media_id"],
            ["media.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_media_hashtag_hashtag"), "media_hashtag", ["hashtag"], unique=False
    )
    op.create_index(
        op.f("ix_media_hashtag_media_id"), "media_hashtag", ["media_id"], unique=False
    )
    op.create_table(
        "media_path",
        sa.Column("id", sa.String(length=32), nullable=False),
        sa.Column("media_id", sa.String(length=32), nullable=False),
        sa.Column("path", sa.String(length=4096), nullable=False),
        sa.ForeignKeyConstraint(
            ["media_id"],
            ["media.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_media_path_media_id"), "media_path", ["media_id"], unique=False
    )
    op.create_index(op.f("ix_media_path_path"), "media_path", ["path"], unique=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_media_path_path"), table_name="media_path")
    op.drop_index(op.f("ix_media_path_media_id"), table_name="media_path")
    op.drop_table("media_path")
    op.drop_index(op.f("ix_media_hashtag_media_id"), table_name="media_hashtag")
    op.drop_index(op.f("ix_media_hashtag_hashtag"), table_name="media_hashtag")
    op.drop_table("media_hashtag")
    op.drop_index(op.f("ix_media_gps_lon"), table_name="media_gps")
    op.drop_index(op.f("ix_media_gps_lat"), table_name="media_gps")
    op.drop_table("media_gps")
    op.drop_index(
        op.f("ix_media_col_item_media_collection_id"), table_name="media_col_item"
    )
    op.drop_index(op.f("ix_media_col_item_media_col_item"), table_name="media_col_item")
    op.drop_table("media_col_item")
    op.drop_table("setting")
    op.drop_index(op.f("ix_media_collection_name"), table_name="media_collection")
    op.drop_index(op.f("ix_media_collection_last_media"), table_name="media_collection")
    op.drop_index(
        op.f("ix_media_collection_first_media"), table_name="media_collection"
    )
    op.drop_table("media_collection")
    op.drop_index(op.f("ix_media_timestamp"), table_name="media")
    op.drop_index(op.f("ix_media_repopath"), table_name="media")
    op.drop_index(op.f("ix_media_hash"), table_name="media")
    op.drop_index(op.f("ix_media_filename"), table_name="media")
    op.drop_table("media")
    # ### end Alembic commands ###
